import type { WritableAtom, SetStateAction } from 'jotai/vanilla';
export type CommonState<Value> = {
    value: Value;
    isDirty: boolean;
};
export type AsyncState<Value> = CommonState<Value> & ({
    isValidating: true;
} | {
    isValidating: false;
    isValid: true;
} | {
    isValidating: false;
    isValid: false;
    error: unknown;
});
export type SyncState<Value> = CommonState<Value> & ({
    isValid: true;
} | {
    isValid: false;
    error: unknown;
});
type CommonOptions<Value> = {
    areEqual?: (a: Value, b: Value) => boolean;
};
type AsyncOptions<Value> = CommonOptions<Value> & {
    validate: (value: Value) => Promise<Value>;
};
type SyncOptions<Value> = CommonOptions<Value> & {
    validate: (value: Value) => Value;
};
export declare function atomWithValidate<Value>(initialValue: Value, options: AsyncOptions<Value>): WritableAtom<AsyncState<Value>, [SetStateAction<Value>], void>;
export declare function atomWithValidate<Value>(initialValue: Value, options: SyncOptions<Value>): WritableAtom<SyncState<Value>, [SetStateAction<Value>], void>;
export {};
